/**********************************************************************
  ChipCard Downloader

  file : main.c

  DATE:    $Date: 1995/11/19 07:18:42 $
  AUTHOR:  $Author: kimoto $
  VERSION: $Revision: 1.2 $

  Copyright (c) 1995 Masahiko Kimoto, All rights reserved.
**********************************************************************/

static char rcsid[]="$Id$";

#include <stdlib.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>

#define DEVFILE "/dev/mc%d"
#define DEFAULT_SLOT 0


extern char *optarg;
extern int optind;
extern int optopt;
extern int opterr;
extern int optreset;

off_t foffset;
int forcemem;

int 
dummy_read(int fd,off_t size)
{
  char buffer[1024];
  int i;
    
  while((int)size > 0) {
    i = ((int)size > 1024?1024:(int)size);
    if (read(fd,buffer,i) <= 0) return -1;
    (int)size -= i;
  }
  return 1;
}


int
seek_write(int fd,void *dat,int size,int offset)
{
  off_t seeks = 0;


  if (offset < 0x8000) {
    offset += 0x20000;
    if (offset < 0x20200 || offset > 0x202ff) {
      printf("Warning : this address is in system area ");
      if (forcemem) 
	printf("\n");
      else {
	printf("(not written.)\n");
	return 0;
      }
    }
  }
  
  if (offset < foffset) {
    /* retrain */
    seeks = offset;
    foffset = 0;
    lseek(fd,0,0);
  } else if (offset > foffset) {
    seeks = offset - foffset;
  }
  if (dummy_read(fd,seeks) < 0) {
    perror("seek");
    return -1;
  }
  foffset = offset + size;

  return write(fd,dat,size);
}


main(int argc,char *argv[])
{
  int fd,c,slot;
  int len,i,addr;
  FILE *fp;
  char *ptr;
  char device[256];
  char buffer[BUFSIZ];
  char tmp[BUFSIZ];
  char buf[BUFSIZ];

  slot = DEFAULT_SLOT;
  forcemem = 0;
  while((c = getopt(argc,argv,"s:fh")) != EOF) {
    switch(c) {
    case 's':
      slot = atoi(optarg);
      break;
    case 'f':
      forcemem = 1;
      break;
    case 'h':
    default :
	    printf("usage : chipldr [-f] [-s num] file\n\t-f     : force to write to system area.\n\t-s num : specify slot number.\n");
      exit(0);
    }
  }
  
  sprintf(device,DEVFILE,slot);
  if ((fd = open(device,O_RDWR)) < 0) {
    perror("open device");
    exit(1);
  }
  foffset = 0;
  
  if (argc > optind) {
    if ((fp = fopen(argv[optind],"r")) == NULL) {
      perror("open file");
      exit(1);
    }
  } else {
    fp = stdin;
  }
  
  buffer[sizeof(buffer)-1] = 0;
  while(feof(fp) == 0) {
    if (fgets(buffer,sizeof(buffer)-1,fp) == NULL) break;
    ptr = buffer;
    
    if (*(ptr) != 'S' || *(ptr+1) != '2') continue;
    bzero(tmp,4);
    bcopy(ptr+2,tmp,2);
    len = strtol(tmp,NULL,16) - 4;
    
    bzero(tmp,8);
    bcopy(ptr+4,tmp,6);
    addr = (strtol(tmp,NULL,16) & 0x1ffff);
    
    bzero(tmp,4);
    for (i=0;i<len;i++) {
      bcopy(ptr+10+i*2,tmp,2);
      buf[i] = strtol(tmp,NULL,16);
    }
/*    lseek(fd,addr,0);*/
    printf("write %04x bytes from %05x\n",len,addr);
    if (seek_write(fd,buf,len,addr) < len) {
      perror("write device");
      exit(1);
    }
  }
}

